@@ -1,12 +1,10 @@
 """
 Python logger for the telnet server.
 """
-from __future__ import unicode_literals
-
 import logging
 
 logger = logging.getLogger(__package__)
 
 __all__ = [
-    'logger',
+    "logger",
 ]
