@@ -5,7 +5,17 @@ import bisect
 import re
 import string
 import weakref
-from typing import Callable, Dict, Iterable, List, Optional, Pattern, Tuple, cast
+from typing import (
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    NoReturn,
+    Optional,
+    Pattern,
+    Tuple,
+    cast,
+)
 
 from .clipboard import ClipboardData
 from .filters import vi_mode
@@ -42,13 +52,13 @@ _text_to_document_cache: Dict[str, "_DocumentCache"] = cast(
 )
 
 
-class _ImmutableLineList(list):
+class _ImmutableLineList(List[str]):
     """
     Some protection for our 'lines' list, which is assumed to be immutable in the cache.
     (Useful for detecting obvious bugs.)
     """
 
-    def _error(self, *a: object, **kw: object) -> None:
+    def _error(self, *a: object, **kw: object) -> NoReturn:
         raise NotImplementedError("Attempt to modify an immutable list.")
 
     __setitem__ = _error  # type: ignore
@@ -59,7 +69,7 @@ class _ImmutableLineList(list):
     pop = _error
     remove = _error
     reverse = _error
-    sort = _error
+    sort = _error  # type: ignore
 
 
 class _DocumentCache:
@@ -96,7 +106,7 @@ class Document:
         # Check cursor position. It can also be right after the end. (Where we
         # insert text.)
         assert cursor_position is None or cursor_position <= len(text), AssertionError(
-            "cursor_position=%r, len_text=%r" % (cursor_position, len(text))
+            f"cursor_position={cursor_position!r}, len_text={len(text)!r}"
         )
 
         # By default, if no cursor position was given, make sure to put the
@@ -127,7 +137,7 @@ class Document:
         # assert self._cache
 
     def __repr__(self) -> str:
-        return "%s(%r, %r)" % (self.__class__.__name__, self.text, self.cursor_position)
+        return f"{self.__class__.__name__}({self.text!r}, {self.cursor_position!r})"
 
     def __eq__(self, other: object) -> bool:
         if not isinstance(other, Document):
@@ -141,27 +151,27 @@ class Document:
 
     @property
     def text(self) -> str:
-        " The document text. "
+        "The document text."
         return self._text
 
     @property
     def cursor_position(self) -> int:
-        " The document cursor position. "
+        "The document cursor position."
         return self._cursor_position
 
     @property
     def selection(self) -> Optional[SelectionState]:
-        " :class:`.SelectionState` object. "
+        ":class:`.SelectionState` object."
         return self._selection
 
     @property
     def current_char(self) -> str:
-        """ Return character under cursor or an empty string. """
+        """Return character under cursor or an empty string."""
         return self._get_char_relative_to_cursor(0) or ""
 
     @property
     def char_before_cursor(self) -> str:
-        """ Return character before the cursor or an empty string. """
+        """Return character before the cursor or an empty string."""
         return self._get_char_relative_to_cursor(-1) or ""
 
     @property
@@ -174,13 +184,13 @@ class Document:
 
     @property
     def current_line_before_cursor(self) -> str:
-        """ Text from the start of the line until the cursor. """
+        """Text from the start of the line until the cursor."""
         _, _, text = self.text_before_cursor.rpartition("\n")
         return text
 
     @property
     def current_line_after_cursor(self) -> str:
-        """ Text from the cursor until the end of the line. """
+        """Text from the cursor until the end of the line."""
         text, _, _ = self.text_after_cursor.partition("\n")
         return text
 
@@ -244,7 +254,7 @@ class Document:
 
     @property
     def leading_whitespace_in_current_line(self) -> str:
-        """ The leading whitespace in the left margin of the current line.  """
+        """The leading whitespace in the left margin of the current line."""
         current_line = self.current_line
         length = len(current_line) - len(current_line.lstrip())
         return current_line[:length]
@@ -341,12 +351,12 @@ class Document:
 
     @property
     def is_cursor_at_the_end(self) -> bool:
-        """ True when the cursor is at the end of the text. """
+        """True when the cursor is at the end of the text."""
         return self.cursor_position == len(self.text)
 
     @property
     def is_cursor_at_the_end_of_line(self) -> bool:
-        """ True when the cursor is at the end of this line. """
+        """True when the cursor is at the end of this line."""
         return self.current_char in ("\n", "")
 
     def has_match_at_current_position(self, sub: str) -> bool:
@@ -456,7 +466,7 @@ class Document:
         self, WORD: bool = False, pattern: Optional[Pattern[str]] = None
     ) -> bool:
         if pattern:
-            return self.find_start_of_previous_word(pattern=pattern) is None
+            return self.find_start_of_previous_word(WORD=WORD, pattern=pattern) is None
         else:
             return (
                 self.text_before_cursor == "" or self.text_before_cursor[-1:].isspace()
@@ -836,15 +846,15 @@ class Document:
         return 0
 
     def get_start_of_document_position(self) -> int:
-        """ Relative position for the start of the document. """
+        """Relative position for the start of the document."""
         return -self.cursor_position
 
     def get_end_of_document_position(self) -> int:
-        """ Relative position for the end of the document. """
+        """Relative position for the end of the document."""
         return len(self.text) - self.cursor_position
 
     def get_start_of_line_position(self, after_whitespace: bool = False) -> int:
-        """ Relative position for the start of this line. """
+        """Relative position for the start of this line."""
         if after_whitespace:
             current_line = self.current_line
             return (
@@ -856,7 +866,7 @@ class Document:
             return -len(self.current_line_before_cursor)
 
     def get_end_of_line_position(self) -> int:
-        """ Relative position for the end of this line. """
+        """Relative position for the end of this line."""
         return len(self.current_line_after_cursor)
 
     def last_non_blank_of_current_line_position(self) -> int:
