@@ -2,7 +2,7 @@
 from typing import Dict, Union
 
 from prompt_toolkit.application.current import get_app
-from prompt_toolkit.buffer import Buffer, SelectionType, indent, unindent
+from prompt_toolkit.buffer import Buffer, indent, unindent
 from prompt_toolkit.completion import CompleteEvent
 from prompt_toolkit.filters import (
     Condition,
@@ -19,6 +19,7 @@ from prompt_toolkit.filters import (
 from prompt_toolkit.key_binding.key_bindings import Binding
 from prompt_toolkit.key_binding.key_processor import KeyPressEvent
 from prompt_toolkit.keys import Keys
+from prompt_toolkit.selection import SelectionType
 
 from ..key_bindings import ConditionalKeyBindings, KeyBindings, KeyBindingsBase
 from .named_commands import get_by_name
@@ -44,7 +45,7 @@ def load_emacs_bindings() -> KeyBindingsBase:
     insert_mode = emacs_insert_mode
 
     @handle("escape")
-    def _(event: E) -> None:
+    def _esc(event: E) -> None:
         """
         By default, ignore escape key.
 
@@ -104,13 +105,13 @@ def load_emacs_bindings() -> KeyBindingsBase:
     handle("c-x", "e")(get_by_name("call-last-kbd-macro"))
 
     @handle("c-n")
-    def _(event: E) -> None:
-        " Next line. "
+    def _next(event: E) -> None:
+        "Next line."
         event.current_buffer.auto_down()
 
     @handle("c-p")
-    def _(event: E) -> None:
-        " Previous line. "
+    def _prev(event: E) -> None:
+        "Previous line."
         event.current_buffer.auto_up(count=event.arg)
 
     def handle_digit(c: str) -> None:
@@ -128,14 +129,13 @@ def load_emacs_bindings() -> KeyBindingsBase:
         handle_digit(c)
 
     @handle("escape", "-", filter=~has_arg)
-    def _(event: E) -> None:
-        """
-        """
+    def _meta_dash(event: E) -> None:
+        """"""
         if event._arg is None:
             event.append_to_arg_count("-")
 
     @handle("-", filter=Condition(lambda: get_app().key_processor.arg == "-"))
-    def _(event: E) -> None:
+    def _dash(event: E) -> None:
         """
         When '-' is typed again, after exactly '-' has been given as an
         argument, ignore this.
@@ -168,36 +168,36 @@ def load_emacs_bindings() -> KeyBindingsBase:
             buff.cursor_position += match
 
     @handle("c-]", Keys.Any)
-    def _(event: E) -> None:
-        " When Ctl-] + a character is pressed. go to that character. "
+    def _goto_char(event: E) -> None:
+        "When Ctl-] + a character is pressed. go to that character."
         # Also named 'character-search'
         character_search(event.current_buffer, event.data, event.arg)
 
     @handle("escape", "c-]", Keys.Any)
-    def _(event: E) -> None:
-        " Like Ctl-], but backwards. "
+    def _goto_char_backwards(event: E) -> None:
+        "Like Ctl-], but backwards."
         # Also named 'character-search-backward'
         character_search(event.current_buffer, event.data, -event.arg)
 
     @handle("escape", "a")
-    def _(event: E) -> None:
-        " Previous sentence. "
+    def _prev_sentence(event: E) -> None:
+        "Previous sentence."
         # TODO:
 
     @handle("escape", "e")
-    def _(event: E) -> None:
-        " Move to end of sentence. "
+    def _end_of_sentence(event: E) -> None:
+        "Move to end of sentence."
         # TODO:
 
     @handle("escape", "t", filter=insert_mode)
-    def _(event: E) -> None:
+    def _swap_characters(event: E) -> None:
         """
         Swap the last two words before the cursor.
         """
         # TODO
 
     @handle("escape", "*", filter=insert_mode)
-    def _(event: E) -> None:
+    def _insert_all_completions(event: E) -> None:
         """
         `meta-*`: Insert all possible completions of the preceding text.
         """
@@ -214,7 +214,7 @@ def load_emacs_bindings() -> KeyBindingsBase:
         buff.insert_text(text_to_insert)
 
     @handle("c-x", "c-x")
-    def _(event: E) -> None:
+    def _toggle_start_end(event: E) -> None:
         """
         Move cursor back and forth between the start and end of the current
         line.
@@ -229,7 +229,7 @@ def load_emacs_bindings() -> KeyBindingsBase:
             buffer.cursor_position += buffer.document.get_end_of_line_position()
 
     @handle("c-@")  # Control-space or Control-@
-    def _(event: E) -> None:
+    def _start_selection(event: E) -> None:
         """
         Start of the selection (if the current buffer is not empty).
         """
@@ -239,7 +239,7 @@ def load_emacs_bindings() -> KeyBindingsBase:
             buff.start_selection(selection_type=SelectionType.CHARACTERS)
 
     @handle("c-g", filter=~has_selection)
-    def _(event: E) -> None:
+    def _cancel(event: E) -> None:
         """
         Control + G: Cancel completion menu and validation state.
         """
@@ -247,7 +247,7 @@ def load_emacs_bindings() -> KeyBindingsBase:
         event.current_buffer.validation_error = None
 
     @handle("c-g", filter=has_selection)
-    def _(event: E) -> None:
+    def _cancel_selection(event: E) -> None:
         """
         Cancel selection.
         """
@@ -255,7 +255,7 @@ def load_emacs_bindings() -> KeyBindingsBase:
 
     @handle("c-w", filter=has_selection)
     @handle("c-x", "r", "k", filter=has_selection)
-    def _(event: E) -> None:
+    def _cut(event: E) -> None:
         """
         Cut selected text.
         """
@@ -263,7 +263,7 @@ def load_emacs_bindings() -> KeyBindingsBase:
         event.app.clipboard.set_data(data)
 
     @handle("escape", "w", filter=has_selection)
-    def _(event: E) -> None:
+    def _copy(event: E) -> None:
         """
         Copy selected text.
         """
@@ -271,7 +271,7 @@ def load_emacs_bindings() -> KeyBindingsBase:
         event.app.clipboard.set_data(data)
 
     @handle("escape", "left")
-    def _(event: E) -> None:
+    def _start_of_word(event: E) -> None:
         """
         Cursor to start of previous word.
         """
@@ -281,7 +281,7 @@ def load_emacs_bindings() -> KeyBindingsBase:
         )
 
     @handle("escape", "right")
-    def _(event: E) -> None:
+    def _start_next_word(event: E) -> None:
         """
         Cursor to start of next word.
         """
@@ -292,7 +292,7 @@ def load_emacs_bindings() -> KeyBindingsBase:
         )
 
     @handle("escape", "/", filter=insert_mode)
-    def _(event: E) -> None:
+    def _complete(event: E) -> None:
         """
         M-/: Complete.
         """
@@ -303,7 +303,7 @@ def load_emacs_bindings() -> KeyBindingsBase:
             b.start_completion(select_first=True)
 
     @handle("c-c", ">", filter=has_selection)
-    def _(event: E) -> None:
+    def _indent(event: E) -> None:
         """
         Indent selected text.
         """
@@ -320,7 +320,7 @@ def load_emacs_bindings() -> KeyBindingsBase:
         indent(buffer, from_, to + 1, count=event.arg)
 
     @handle("c-c", "<", filter=has_selection)
-    def _(event: E) -> None:
+    def _unindent(event: E) -> None:
         """
         Unindent selected text.
         """
@@ -381,8 +381,8 @@ def load_emacs_search_bindings() -> KeyBindingsBase:
     )
 
     @handle("n", filter=is_read_only)
-    def _(event: E) -> None:
-        " Jump to next match. "
+    def _jump_next(event: E) -> None:
+        "Jump to next match."
         event.current_buffer.apply_search(
             event.app.current_search_state,
             include_current_position=False,
@@ -390,8 +390,8 @@ def load_emacs_search_bindings() -> KeyBindingsBase:
         )
 
     @handle("N", filter=is_read_only)
-    def _(event: E) -> None:
-        " Jump to previous match. "
+    def _jump_prev(event: E) -> None:
+        "Jump to previous match."
         event.current_buffer.apply_search(
             ~event.app.current_search_state,
             include_current_position=False,
@@ -430,22 +430,21 @@ def load_emacs_shift_selection_bindings() -> KeyBindingsBase:
             Keys.ShiftRight: "forward-char",
             Keys.ShiftHome: "beginning-of-line",
             Keys.ShiftEnd: "end-of-line",
-            Keys.ShiftControlLeft: "backward-word",
-            Keys.ShiftControlRight: "forward-word",
-            Keys.ShiftControlHome: "beginning-of-buffer",
-            Keys.ShiftControlEnd: "end-of-buffer",
+            Keys.ControlShiftLeft: "backward-word",
+            Keys.ControlShiftRight: "forward-word",
+            Keys.ControlShiftHome: "beginning-of-buffer",
+            Keys.ControlShiftEnd: "end-of-buffer",
         }
 
         try:
             # Both the dict lookup and `get_by_name` can raise KeyError.
-            handler = get_by_name(key_to_command[key])
+            binding = get_by_name(key_to_command[key])
         except KeyError:
             pass
         else:  # (`else` is not really needed here.)
-            if not isinstance(handler, Binding):
-                # (It should always be a normal callable here, for these
-                # commands.)
-                handler(event)
+            if isinstance(binding, Binding):
+                # (It should always be a binding here)
+                binding.call(event)
 
     @handle("s-left", filter=~has_selection)
     @handle("s-right", filter=~has_selection)
@@ -453,11 +452,11 @@ def load_emacs_shift_selection_bindings() -> KeyBindingsBase:
     @handle("s-down", filter=~has_selection)
     @handle("s-home", filter=~has_selection)
     @handle("s-end", filter=~has_selection)
-    @handle("s-c-left", filter=~has_selection)
-    @handle("s-c-right", filter=~has_selection)
-    @handle("s-c-home", filter=~has_selection)
-    @handle("s-c-end", filter=~has_selection)
-    def _(event: E) -> None:
+    @handle("c-s-left", filter=~has_selection)
+    @handle("c-s-right", filter=~has_selection)
+    @handle("c-s-home", filter=~has_selection)
+    @handle("c-s-end", filter=~has_selection)
+    def _start_selection(event: E) -> None:
         """
         Start selection with shift + movement.
         """
@@ -465,7 +464,12 @@ def load_emacs_shift_selection_bindings() -> KeyBindingsBase:
         buff = event.current_buffer
         if buff.text:
             buff.start_selection(selection_type=SelectionType.CHARACTERS)
-            buff.selection_state.enter_shift_mode()
+
+            if buff.selection_state is not None:
+                # (`selection_state` should never be `None`, it is created by
+                # `start_selection`.)
+                buff.selection_state.enter_shift_mode()
+
             # Then move the cursor
             original_position = buff.cursor_position
             unshift_move(event)
@@ -480,31 +484,33 @@ def load_emacs_shift_selection_bindings() -> KeyBindingsBase:
     @handle("s-down", filter=shift_selection_mode)
     @handle("s-home", filter=shift_selection_mode)
     @handle("s-end", filter=shift_selection_mode)
-    @handle("s-c-left", filter=shift_selection_mode)
-    @handle("s-c-right", filter=shift_selection_mode)
-    @handle("s-c-home", filter=shift_selection_mode)
-    @handle("s-c-end", filter=shift_selection_mode)
-    def _(event: E) -> None:
+    @handle("c-s-left", filter=shift_selection_mode)
+    @handle("c-s-right", filter=shift_selection_mode)
+    @handle("c-s-home", filter=shift_selection_mode)
+    @handle("c-s-end", filter=shift_selection_mode)
+    def _extend_selection(event: E) -> None:
         """
         Extend the selection
         """
         # Just move the cursor, like shift was not pressed
         unshift_move(event)
         buff = event.current_buffer
-        if buff.cursor_position == buff.selection_state.original_cursor_position:
-            # selection is now empty, so cancel selection
-            buff.exit_selection()
+
+        if buff.selection_state is not None:
+            if buff.cursor_position == buff.selection_state.original_cursor_position:
+                # selection is now empty, so cancel selection
+                buff.exit_selection()
 
     @handle(Keys.Any, filter=shift_selection_mode)
-    def _(event: E) -> None:
+    def _replace_selection(event: E) -> None:
         """
         Replace selection by what is typed
         """
         event.current_buffer.cut_selection()
-        get_by_name("self-insert")(event)
+        get_by_name("self-insert").call(event)
 
     @handle("enter", filter=shift_selection_mode & is_multiline)
-    def _(event: E) -> None:
+    def _newline(event: E) -> None:
         """
         A newline replaces the selection
         """
@@ -512,21 +518,21 @@ def load_emacs_shift_selection_bindings() -> KeyBindingsBase:
         event.current_buffer.newline(copy_margin=not in_paste_mode())
 
     @handle("backspace", filter=shift_selection_mode)
-    def _(event: E) -> None:
+    def _delete(event: E) -> None:
         """
         Delete selection.
         """
         event.current_buffer.cut_selection()
 
     @handle("c-y", filter=shift_selection_mode)
-    def _(event: E) -> None:
+    def _yank(event: E) -> None:
         """
         In shift selection mode, yanking (pasting) replace the selection.
         """
         buff = event.current_buffer
         if buff.selection_state:
             buff.cut_selection()
-        get_by_name("yank")(event)
+        get_by_name("yank").call(event)
 
     # moving the cursor in shift selection mode cancels the selection
     @handle("left", filter=shift_selection_mode)
@@ -539,7 +545,7 @@ def load_emacs_shift_selection_bindings() -> KeyBindingsBase:
     @handle("c-right", filter=shift_selection_mode)
     @handle("c-home", filter=shift_selection_mode)
     @handle("c-end", filter=shift_selection_mode)
-    def _(event: E) -> None:
+    def _cancel(event: E) -> None:
         """
         Cancel selection.
         """
